@@ -104,13 +104,14 @@ pub struct Target {
     poll: Cell<usize>,
     config_data: RefCell<Option<PendingAction>>,
     deployment: RefCell<Option<PendingAction>>,
+    cancel_action: RefCell<Option<PendingAction>>,
 }
 
 impl Target {
     fn new(name: &str, server: &Rc<MockServer>, tenant: &str) -> Self {
         let key = format!("Key{}", name);
 
-        let poll = Self::create_poll(server, tenant, name, &key, None, None);
+        let poll = Self::create_poll(server, tenant, name, &key, None, None, None);
         Target {
             name: name.to_string(),
             key,
@@ -119,6 +120,7 @@ impl Target {
             poll: Cell::new(poll),
             config_data: RefCell::new(None),
             deployment: RefCell::new(None),
+            cancel_action: RefCell::new(None),
         }
     }
 
@@ -129,6 +131,7 @@ impl Target {
         key: &str,
         expected_config_data: Option<&PendingAction>,
         deployment: Option<&PendingAction>,
+        cancel_action: Option<&PendingAction>,
     ) -> usize {
         let mut links = Map::new();
 
@@ -138,6 +141,9 @@ impl Target {
         if let Some(pending) = deployment {
             links.insert("deploymentBase".into(), json!({ "href": pending.path }));
         }
+        if let Some(pending) = cancel_action {
+            links.insert("cancelAction".into(), json!({ "href": pending.path }));
+        }
 
         let response = json!({
             "config": {
@@ -169,6 +175,7 @@ impl Target {
             &self.key,
             self.config_data.borrow().as_ref(),
             self.deployment.borrow().as_ref(),
+            self.cancel_action.borrow().as_ref(),
         ));
 
         let mut old = MockRef::new(old, &self.server);
@@ -237,7 +244,7 @@ impl Target {
     ///
     /// ```
     /// use std::path::Path;
-    /// use hawkbit_mock::ddi::{ServerBuilder, DeploymentBuilder};
+    /// use hawkbit_mock::ddi::{ChunkProtocol, ServerBuilder, DeploymentBuilder};
     /// use hawkbit::ddi::{Type, MaintenanceWindow};
     ///
     /// let server = ServerBuilder::default().build();
@@ -246,6 +253,7 @@ impl Target {
     /// let deployment = DeploymentBuilder::new("10", Type::Forced, Type::Attempt)
     ///    .maintenance_window(MaintenanceWindow::Available)
     ///    .chunk(
+    ///       ChunkProtocol::BOTH,
     ///       "app",
     ///       "1.0",
     ///        "some-chunk",
@@ -309,7 +317,7 @@ impl Target {
         self.update_poll();
     }
 
-    /// Configure the server to expect feedback from the target.
+    /// Configure the server to expect deployment feedback from the target.
     /// One can then check the feedback has actually been received using
     /// `hits()` on the returned object.
     ///
@@ -322,7 +330,7 @@ impl Target {
     ///
     /// let server = ServerBuilder::default().build();
     /// let target = server.add_target("Target1");
-    /// let mut mock = target.expect_feedback(
+    /// let mut mock = target.expect_deployment_feedback(
     ///         "10",
     ///         Execution::Closed,
     ///         Finished::Success,
@@ -334,7 +342,7 @@ impl Target {
     /// //Client send the feedback
     /// //assert_eq!(mock.hits(), 1);
     /// ```
-    pub fn expect_feedback(
+    pub fn expect_deployment_feedback(
         &self,
         deployment_id: &str,
         execution: Execution,
@@ -342,14 +350,133 @@ impl Target {
         progress: Option<serde_json::Value>,
         details: Vec<&str>,
     ) -> MockRef<'_> {
-        let progress = progress.unwrap_or(serde_json::Value::Null);
+        self.server.mock(|when, then| {
+            let expected = match progress {
+                Some(progress) => json!({
+                    "id": deployment_id,
+                    "status": {
+                        "result": {
+                            "progress": progress,
+                            "finished": finished
+                        },
+                        "execution": execution,
+                        "details": details,
+                    },
+                }),
+                None => json!({
+                    "id": deployment_id,
+                    "status": {
+                        "result": {
+                            "finished": finished
+                        },
+                        "execution": execution,
+                        "details": details,
+                    },
+                }),
+            };
 
+            when.method(POST)
+                .path(format!(
+                    "/{}/controller/v1/{}/deploymentBase/{}/feedback",
+                    self.tenant, self.name, deployment_id
+                ))
+                .header("Authorization", &format!("TargetToken {}", self.key))
+                .header("Content-Type", "application/json")
+                .json_body(expected);
+
+            then.status(200);
+        })
+    }
+
+    /// Push a cancel action update to the target.
+    /// One can then use [`Target::cancel_action_hits`] to check that the client
+    /// fetched the details about the cancel action.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use hawkbit_mock::ddi::ServerBuilder;
+    ///
+    /// let server = ServerBuilder::default().build();
+    /// let target = server.add_target("Target1");
+    /// target.cancel_action("5");
+    ///
+    /// // Client fetches details about the cancel action
+    /// //assert_eq!(target.cancel_action_hits(), 1);
+    /// ```
+    pub fn cancel_action(&self, id: &str) {
+        let cancel_path = self.server.url(format!(
+            "/DEFAULT/controller/v1/{}/cancelAction/{}",
+            self.name, id
+        ));
+
+        let response = json!({
+            "id": id,
+            "cancelAction": {
+                "stopId": id
+            }
+        });
+
+        let cancel_mock = self.server.mock(|when, then| {
+            when.method(GET)
+                .path(format!(
+                    "/DEFAULT/controller/v1/{}/cancelAction/{}",
+                    self.name, id
+                ))
+                .header("Authorization", &format!("TargetToken {}", self.key));
+
+            then.status(200)
+                .header("Content-Type", "application/json")
+                .json_body(response);
+        });
+
+        self.cancel_action.replace(Some(PendingAction {
+            server: self.server.clone(),
+            path: cancel_path,
+            mock: cancel_mock.id(),
+        }));
+
+        self.update_poll();
+    }
+
+    /// Configure the server to expect cancel feedback from the target.
+    /// One can then check the feedback has actually been received using
+    /// `hits()` on the returned object.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use hawkbit_mock::ddi::{ServerBuilder, DeploymentBuilder};
+    /// use hawkbit::ddi::{Execution, Finished};
+    /// use serde_json::json;
+    ///
+    /// let server = ServerBuilder::default().build();
+    /// let target = server.add_target("Target1");
+    /// target.cancel_action("10");
+    ///
+    /// let mut mock = target.expect_cancel_feedback(
+    ///         "10",
+    ///         Execution::Closed,
+    ///         Finished::Success,
+    ///         vec!["Cancelled"],
+    ///     );
+    /// assert_eq!(mock.hits(), 0);
+    ///
+    /// //Client send the feedback
+    /// //assert_eq!(mock.hits(), 1);
+    /// ```
+    pub fn expect_cancel_feedback(
+        &self,
+        cancel_id: &str,
+        execution: Execution,
+        finished: Finished,
+        details: Vec<&str>,
+    ) -> MockRef<'_> {
         self.server.mock(|when, then| {
             let expected = json!({
-                "id": deployment_id,
+                "id": cancel_id,
                 "status": {
                     "result": {
-                        "progress": progress,
                         "finished": finished
                     },
                     "execution": execution,
@@ -359,8 +486,8 @@ impl Target {
 
             when.method(POST)
                 .path(format!(
-                    "/{}/controller/v1/{}/deploymentBase/{}/feedback",
-                    self.tenant, self.name, deployment_id
+                    "/{}/controller/v1/{}/cancelAction/{}/feedback",
+                    self.tenant, self.name, cancel_id
                 ))
                 .header("Authorization", &format!("TargetToken {}", self.key))
                 .header("Content-Type", "application/json")
@@ -391,6 +518,14 @@ impl Target {
             mock.hits()
         })
     }
+
+    /// Return the number of times the cancel action URL has been fetched by the client.
+    pub fn cancel_action_hits(&self) -> usize {
+        self.cancel_action.borrow().as_ref().map_or(0, |m| {
+            let mock = MockRef::new(m.mock, &self.server);
+            mock.hits()
+        })
+    }
 }
 
 struct PendingAction {
@@ -414,6 +549,7 @@ pub struct DeploymentBuilder {
     maintenance_window: Option<MaintenanceWindow>,
     chunks: Vec<Chunk>,
 }
+
 /// A pending deployment update pushed to the target.
 pub struct Deployment {
     /// The id of the deployment
@@ -445,6 +581,7 @@ impl DeploymentBuilder {
 
     /// Add a new software chunk to the deployment.
     /// # Arguments
+    /// * `protocol`: The protocols over which chunks are downloadable
     /// * `part`: the type of chunk, e.g. `firmware`, `bundle`, `app`
     /// * `version`: software version of the chunk
     /// * `name`: name of the chunk
@@ -455,6 +592,7 @@ impl DeploymentBuilder {
     ///   * the `sha256sum` of the file.
     pub fn chunk(
         self,
+        protocol: ChunkProtocol,
         part: &str,
         version: &str,
         name: &str,
@@ -471,6 +609,7 @@ impl DeploymentBuilder {
             .collect();
 
         let chunk = Chunk {
+            protocol,
             part: part.to_string(),
             version: version.to_string(),
             name: name.to_string(),
@@ -493,8 +632,31 @@ impl DeploymentBuilder {
     }
 }
 
+/// Protocol(s) over which chunks are served
+pub enum ChunkProtocol {
+    /// both http and https
+    BOTH,
+    /// http only
+    HTTP,
+    /// https only
+    HTTPS,
+}
+
+impl ChunkProtocol {
+    /// Return whether the http protocol is used for downloads
+    pub fn http(&self) -> bool {
+        matches!(self, Self::BOTH | Self::HTTP)
+    }
+
+    /// Return whether the https protocol is used for downloads
+    pub fn https(&self) -> bool {
+        matches!(self, Self::BOTH | Self::HTTPS)
+    }
+}
+
 /// Software chunk of an update.
 pub struct Chunk {
+    protocol: ChunkProtocol,
     part: String,
     version: String,
     name: String,
@@ -513,6 +675,17 @@ impl Chunk {
                 // TODO: the md5 url is not served by the http server
                 let md5_url = format!("{}.MD5SUM", download_url);
 
+                let mut links = serde_json::Map::new();
+
+                if self.protocol.https() {
+                    links.insert("download".to_string(), json!({ "href": download_url }));
+                    links.insert("md5sum".to_string(), json!({ "href": md5_url }));
+                }
+                if self.protocol.http() {
+                    links.insert("download-http".to_string(), json!({ "href": download_url }));
+                    links.insert("md5sum-http".to_string(), json!({ "href": md5_url }));
+                }
+
                 json!({
                     "filename": file_name,
                     "hashes": {
@@ -521,20 +694,7 @@ impl Chunk {
                         "sha256": sha256,
                     },
                     "size": meta.len(),
-                    "_links": {
-                        "download": {
-                            "href": download_url,
-                        },
-                        "download-http": {
-                            "href": download_url,
-                        },
-                        "md5sum": {
-                            "href": md5_url,
-                        },
-                        "md5sum-http": {
-                            "href": md5_url,
-                        },
-                    }
+                    "_links": links,
                 })
             })
             .collect();
